from matplotlib import pyplot as plt
from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg


class MplCanvas(FigureCanvasQTAgg):
    def __init__(self, parent=None, width=6, height=4, dpi=100, xlabel='', ylabel='', title='', rotate_xlabel=False):
        """
        Initialize the canvas.

        Parameters
        ----------
        parent : QWidget
            The parent widget.
        width : int
            The width of the canvas.
        height : int
            The height of the canvas.
        dpi : int
            The resolution of the canvas.
        xlabel : str
            The label of the x-axis.
        ylabel : str
            The label of the y-axis.
        title : str
            The title of the canvas.
        rotate_xlabel : bool
            Whether to rotate the x-axis label.
        """
        fig, self.ax = plt.subplots(figsize=(width, height), dpi=dpi)
        if rotate_xlabel:
            plt.xticks(rotation=90)
        super().__init__(fig)
        self.setParent(parent)
        self.ax.set(xlabel=xlabel, ylabel=ylabel, title=title)



from .Ui_clone import *
from .Ui_create_repository import *
from .Ui_edit_profile import *
from .Ui_edit_repository import *
from .Ui_file_template import *
from .Ui_issue_template import *
from .Ui_local_repositories import *
from .Ui_local_repository import *
from .Ui_local_repository_template import *
from .Ui_overview import *
from .Ui_popular_repositories import *
from .Ui_repositories import *
from .Ui_repository import *
from .Ui_repository_template import *



from PySide6.QtCore import QSize
from PySide6.QtGui import QPixmap, QIcon
from PySide6.QtWidgets import QWidget
from github.AuthenticatedUser import AuthenticatedUser
from github.Repository import Repository

from pygithubmanager.ui.widgets import Ui_RepositoryTemplate
from pygithubmanager.utils import get_icon, time_formatter


class RepositoryTemplate(QWidget):
    def __init__(self, repo: Repository, user: AuthenticatedUser):
        """
        Initialize the template of the repo

        Parameters
        ----------
        repo : Repository
            The repo to display
        user : AuthenticatedUser
            The user who owns the repo
        """
        super(RepositoryTemplate, self).__init__()
        self.ui = Ui_RepositoryTemplate()
        self.ui.setupUi(self)

        self.repo: Repository = repo
        self.user: AuthenticatedUser = user

        self.config_ui()

        self.fill()

    def fill(self):
        """
        Fill the template with the data from the repo
        """
        self.ui.repoNameLabel.setText(self.repo.name)
        self.ui.visibilityLabel.setText('Private' if self.repo.private else 'Public')
        if self.repo.fork:
            self.ui.forkWidget.set_icon("fa.code-fork")
            self.ui.forkWidget.setText("Forked from " + self.repo.parent.full_name)
            self.ui.forkWidget.setStyleSheet("font-size: 9px;")
            self.ui.forkWidget.show()

        self.ui.repoDescLabel.setText(self.repo.description)
        try:
            self.ui.licenseNameLabel.setText(self.repo.get_license().license.name)
        except Exception:
            self.ui.licenseNameLabel.setText("None")
        self.ui.updatedLabel.setText(time_formatter("Updated", time2=self.repo.updated_at))
        self.ui.updatedLabel.setToolTip(self.repo.updated_at.strftime("%d-%m-%Y %H:%M:%S") + " UTC")
        self.ui.starButton.setText("Star" if self.repo.stargazers_count == 0 else "Starred")

    def config_ui(self):
        """
        Configure the UI
        """
        self.ui.licenseIconLabel.setPixmap(QPixmap(get_icon("dark_license.png")).scaled(QSize(15, 15)))
        self.ui.starButton.setIcon(
            QIcon(get_icon("dark_star.png" if self.repo.stargazers_count == 0 else "yellow_star.png")))
        self.ui.visibilityLabel.setStyleSheet("border: 1px solid black; border-radius: 10px; padding: 1px;")
        self.ui.starButton.clicked.connect(self.star_button)
        self.ui.forkWidget.hide()

    def star(self):
        """
        Star the repo
        """
        self.ui.starButton.setText("Starred" if self.repo in self.user.get_starred() else "Star")
        self.ui.starButton.setIcon(
            QIcon(get_icon("yellow_star.png" if self.repo in self.user.get_starred() else "dark_star.png")))

    def star_button(self):
        """
        Star the repo
        """
        if self.repo in self.user.get_starred():
            self.user.remove_from_starred(self.repo)
        else:
            self.user.add_to_starred(self.repo)
        self.star()



from PySide6.QtWidgets import QWidget
from github import GithubException

from pygithubmanager.ui.widgets import Ui_RepositoryEdit
from pygithubmanager.utils import delete_repository_dialog, message


class EditRepository(QWidget):
    def __init__(self, repo, repo_widget):
        """
        EditRepository class constructor

        Parameters
        ----------
        repo : Repository
            Repository to edit
        repo_widget : RepositoryWidget
            RepositoryWidget to close when editing is done
        """
        super(EditRepository, self).__init__()

        self.ui = Ui_RepositoryEdit()
        self.ui.setupUi(self)

        self.repo = repo
        self.repo_widget = repo_widget

        self.fill()
        self.config()

    def fill(self):
        """
        Fill the widgets with the repository data
        """
        self.ui.nameLineEdit.setText(self.repo.name)
        self.ui.descriptionLineEdit.setText(self.repo.description)
        self.ui.homePageLineEdit.setText(self.repo.homepage)
        self.ui.privateCheckBox.setChecked(self.repo.private)
        for branch in self.repo.get_branches():
            self.ui.defaultBranchComboBox.addItem(branch.name)
        self.ui.defaultBranchComboBox.setCurrentText(self.repo.default_branch)

    def config(self):
        """
        Configure the widgets
        """
        self.ui.acceptButton.clicked.connect(self.accept)
        self.ui.cancelButton.clicked.connect(lambda: self.close())
        self.ui.deleteButton.clicked.connect(self.delete)

    def accept(self):
        """
        Accept the changes
        """
        self.repo.edit(
            name=self.ui.nameLineEdit.text(),
            description=self.ui.descriptionLineEdit.text(),
            homepage=self.ui.homePageLineEdit.text(),
            private=self.ui.privateCheckBox.isChecked(),
            default_branch=self.ui.defaultBranchComboBox.currentText(),
        )
        self.close()

    def delete(self):
        """
        Delete the repository

        Raises
        ------
        Exception
            If there is an error deleting the repository
        """
        if delete_repository_dialog('remote'):
            try:
                self.repo.delete()
                message('success', 'Repository deleted')
                self.repo_widget.close()
                self.close()
            except GithubException as e:
                message('error', e.data['message'])



from git import Repo


def get_repo_info(repo_path):
    """
    Obtain information about the repository.

    Parameters
    ----------
    repo_path : str
        Path to the repository.

    Returns
    -------
    repo_info : tuple
        A tuple containing the following information: (username, project_name, full_repo_name)

    Examples
    --------
    >>> get_repo_info('/home/user/repo')
    ('user', 'repo', 'user/repo')
    >>> get_repo_info('~/repo')
    ('user', 'repo', 'user/repo')
    """

    repo = Repo(repo_path)

    repo_name = repo.remotes.origin.url.split('.git')[0].split('/')[-1]
    repo_user = repo.remotes.origin.url.split('.git')[0].split('/')[-2]
    if ":" in repo_user:
        repo_user = repo_user.split(':')[-1]
    full_repo_name = f'{repo_user}/{repo_name}'

    return repo_user, repo_name, full_repo_name



from .CloneTemplate import *
from .CustomWebEnginePage import *
from .EditProfile import *
from .EditRepository import *
from .FileTemplate import *
from .HyperlinkLabel import *
from .IconLabel import *
from .IssueTemplate import *
from .LocalRepositoryListWidgetItem import *
from .MplCanvas import *
from .RepositoryListWidgetItem import *
from .RepositoryTemplate import *
from .RepositoriesListWidgetItem import *



from datetime import datetime


def time_formatter(text, time1=datetime.utcnow(), time2=datetime.utcnow(), suffix="ago"):
    """
    Calculate the time difference between two times and return the result in human-readable format.

    Parameters
    ----------
    text : str
        The text to be displayed before the time.
    time1 : datetime
        The first time to be compared.
    time2 : datetime
        The second time to be compared.
    suffix : str
        The suffix to be appended to the time.

    Returns
    -------
    str
        The time difference in human-readable format.

    Examples
    --------
    >>> time_formatter("Last updated: ", datetime(2020, 1, 1), datetime(2020, 1, 2), "ago")
    'Last updated: 1 day ago'
    >>> time_formatter("Reset in: ", datetime(2020, 1, 1), datetime(2020, 1, 3), "")
    'Reset in: 2 days'
    """
    now = time1
    last = time2

    date = now - last

    txt = text + ' {0} {1} ' + suffix

    if date.days <= 0:
        if date.seconds // 3600 <= 0:
            if (date.seconds // 60) % 60 <= 0:
                txt = txt.format(date.seconds, "seconds" if date.seconds != 1 else "second")
            else:
                txt = txt.format((date.seconds // 60) % 60, "minutes" if (date.seconds // 60) % 60 != 1 else "minute")
        else:
            txt = txt.format(date.seconds // 3600, "hours" if date.seconds // 3600 != 1 else "hour")
    else:
        txt = txt.format(date.days, "days" if date.days != 1 else "day")

    return txt



import os
import sys
import traceback

import qtawesome as qta
from PySide6.QtCore import QRunnable, Signal, Slot, QThreadPool, QObject
from PySide6.QtWidgets import QWidget, QFileDialog

from pygithubmanager.modules.local_repository import LocalRepository
from pygithubmanager.ui.widgets import Ui_LocalRepositories
from pygithubmanager.utils import USER_HOME_PATH
from pygithubmanager.widgets import LocalRepositoryListWidgetItem
from pygithubmanager.widgets.LocalRepositoryTemplate import LocalRepositoryTemplate


class WorkerSignals(QObject):
    repo = Signal(str, str)
    finished = Signal()
    error = Signal(tuple)


class Worker(QRunnable):
    def __init__(self, path):
        """
        Worker to find all local repositories in a given path and emit them asynchronously

        Parameters
        ----------
        path : str
            Path to search for local repositories
        """
        super(Worker, self).__init__()
        self.path = path
        self.signals = WorkerSignals()

    @Slot(str)
    def run(self):
        """
        Find all local repositories in a given path and emit them asynchronously
        """
        try:
            for root, subdirs, files in os.walk(self.path):
                for d in subdirs:
                    if os.access(root, os.R_OK):
                        if d == ".git":
                            folder_name = root.split(os.sep)[-1]
                            self.signals.repo.emit(folder_name, root)
                    else:
                        print(d + " No permission")
        except Exception:
            traceback.print_exc()
            exctype, value = sys.exc_info()[:2]
            self.signals.error.emit((exctype, value, traceback.format_exc()))
        finally:
            self.signals.finished.emit()


class LocalRepositories(QWidget):
    def __init__(self, user):
        """
        Initialize the local repositories widget

        Parameters
        ----------
        user : User
            User object
        """
        super(LocalRepositories, self).__init__()
        self.ui = Ui_LocalRepositories()
        self.ui.setupUi(self)

        self.user = user
        self.open_repo = None

        self.thread_pool = QThreadPool()

        self.config()
        self.thread_load_items()

    def load_finished(self):
        """
        Function to be executed when the thread is finished to enable some UI elements
        """
        self.ui.searchButton.setEnabled(True)
        self.ui.searchPathButton.setEnabled(True)
        self.ui.lineEdit.setEnabled(True)
        self.ui.loadingButton.hide()

    @Slot(str, str)
    def add_new_repo(self, folder_name, path):
        """
        Add a new repository to the list of local repositories

        Parameters
        ----------
        folder_name : str
            Name of the folder containing the repository
        path : str
            Path to the folder containing the repository
        """
        try:
            item = LocalRepositoryListWidgetItem(folder_name, path)
            widget = LocalRepositoryTemplate(path, self.user)
            item.setSizeHint(widget.sizeHint())
            self.ui.listWidget.addItem(item)
            self.ui.listWidget.setItemWidget(item, widget)
        except Exception:
            print("Error while adding new repo " + folder_name + " " + path)

    def thread_load_items(self):
        """
        Load all local repositories in a separate thread
        """
        self.ui.loadingButton.show()
        self.ui.searchButton.setDisabled(True)
        self.ui.searchPathButton.setDisabled(True)
        self.ui.lineEdit.setDisabled(True)
        self.ui.listWidget.clear()

        worker = Worker(self.ui.lineEdit_2.text())
        worker.signals.repo.connect(self.add_new_repo)
        worker.signals.finished.connect(self.load_finished)

        self.thread_pool.start(worker)

    def config(self):
        """
        Configures the signals of the widget
        """
        self.ui.listWidget.itemClicked.connect(self.open_repository)
        self.ui.searchPathButton.clicked.connect(self.save_path)
        self.ui.searchButton.clicked.connect(self.thread_load_items)
        self.ui.lineEdit_2.setText(str(USER_HOME_PATH))
        self.ui.lineEdit.textChanged.connect(self.on_text_changed)
        self.ui.loadingButton.setIcon(qta.icon('ri.loader-4-line', animation=qta.Pulse(self.ui.loadingButton)))
        self.ui.loadingButton.setToolTip("Searching repositories...")
        self.ui.loadingButton.hide()

    def save_path(self):
        """
        Repository save path and update a line edit
        """
        new_route = QFileDialog.getExistingDirectory(self, "Select folder", str(USER_HOME_PATH))
        if new_route:
            self.ui.lineEdit_2.setText(new_route)

    def open_repository(self, item):
        """
        Opens the repository selected in a new window
        """
        repo_name = item.path
        self.open_repo = LocalRepository(repo_name, self.user)
        self.open_repo.show()

    def on_text_changed(self, text):
        """
        Filters the repositories with the text entered the lineEdit
        """
        for row in range(self.ui.listWidget.count()):
            item = self.ui.listWidget.item(row)
            widget = self.ui.listWidget.itemWidget(item)
            if text:
                item.setHidden(not widget.repo_name.lower().startswith(text.lower()))
            else:
                item.setHidden(False)



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QWidget

from pygithubmanager.modules.repository import Repository
from pygithubmanager.ui.widgets import Ui_PopularRepositories


class PopularRepository(QWidget):
    def __init__(self, repository, user):
        """
        Popular repository constructor

        Parameters
        ----------
        repository : github.Repository.Repository
            The repository to display
        user
        """
        super(PopularRepository, self).__init__()
        self.ui = Ui_PopularRepositories()
        self.ui.setupUi(self)

        self.repo = repository
        self.user = user

        self.open_repo_window = None

        self.config()

        self.fill()

    def fill(self):
        """
        Fill the template with the data from the repo
        """
        self.ui.nameLabel.setText(self.repo.name)
        self.ui.visibilityLabel.setText('Private' if self.repo.private else 'Public')
        if self.repo.fork:
            self.ui.forkWidget.set_icon("fa.code-fork")
            self.ui.forkWidget.setText("Forked from " + self.repo.parent.full_name)
            self.ui.forkWidget.setStyleSheet("font-size: 9px;")
            self.ui.forkWidget.show()
        else:
            self.ui.forkWidget.hide()

        self.ui.descLabel.setText(self.repo.description)
        try:
            lang = list(self.repo.get_languages().keys())[0]
            self.ui.langWidget.setText(lang)
            try:
                self.ui.langWidget.set_icon("mdi.language-" + lang.lower())
            except Exception:
                if lang.lower() == "shell":
                    self.ui.langWidget.set_icon("msc.terminal-bash")
                else:
                    self.ui.langWidget.set_icon("fa.github-alt")
        except Exception:
            self.ui.langWidget.hide()

        if self.repo.stargazers_count > 0:
            self.ui.starsWidget.setText(str(self.repo.stargazers_count))
            self.ui.starsWidget.set_icon("fa.star")
        else:
            self.ui.starsWidget.hide()

    def config(self):
        """
        Configure the template
        """
        self.ui.visibilityLabel.setStyleSheet("border: 1px solid black; border-radius: 10px; padding: 1px;")
        self.ui.nameLabel.clicked.connect(self.open_repo)
        self.ui.nameLabel.setStyleSheet("color: blue;")
        self.ui.nameLabel.setCursor(Qt.PointingHandCursor)
        self.ui.frame.setStyleSheet(".QFrame{border: 1px solid black; border-radius: 10px;}")

    def open_repo(self):
        """
        Open the repository in a new window
        """
        self.open_repo_window = Repository(self.repo, self.user)
        self.open_repo_window.show()



import sys
from datetime import datetime
from importlib.metadata import version

import qtawesome as qta
import requests
from PySide6.QtCore import QSize, QTimer
from PySide6.QtGui import QIcon, QPixmap, QImage, Qt, QScreen
from PySide6.QtWidgets import QMainWindow, QApplication, QVBoxLayout, QMessageBox, QWidget
from pygithubmanager.modules import LocalRepositories
from pygithubmanager.modules.about import About
from pygithubmanager.modules.login import Login
from pygithubmanager.ui import Ui_MainWindow
from pygithubmanager.utils import time_formatter
from pygithubmanager.utils.resources import LOGO

from pygithubmanager.modules.overview import Overview
from pygithubmanager.modules.repositories import Repositories
from pygithubmanager.widgets import EditProfile

try:
    from ctypes import windll  # Only exists on Windows.

    myappid = 'xyz.domitp.pygithubmanager.' + version("pygithubmanager")
    windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
except ImportError:
    pass


class GitHubManager(QMainWindow):
    def __init__(self):
        """
        Initializes the main window.

        Sets the main window's UI, and loads the login widget.
        """
        super(GitHubManager, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.login = Login(self)
        self.login.show()

        self.about = About()

        self.edit = QWidget()

        self.user = None

        self.timer = QTimer(self)

    def start(self, user):
        """
        Loads the user's information and repositories.

        When a user logs in with their GitHub token, the main program is started and all the necessary information
        is loaded.

        Parameters
        ----------
        user : modules.user.User
            The user class containing all the user's information.
        """
        self.user = user

        self.config()

        self.load_user_info()
        self.load_repositories()
        self.load_local_repositories()
        self.load_overview()

        self.show()

    def config(self):
        """
        Configures the main window.

        Sets the window's icon, widget connections, and the timer.
        """
        self.setWindowIcon(QIcon(LOGO))

        self.ui.actionExit.triggered.connect(self.close)
        self.ui.actionAbout.triggered.connect(lambda x: self.about.show())
        self.ui.actionReload.triggered.connect(self.reload)
        self.ui.infoButton.clicked.connect(self.requests_info)
        self.ui.editProfileButton.clicked.connect(self.edit_profile)
        self.ui.editProfileButton.hide()  # Actually, edit profile is broken in PyGithub, so it's hidden.

        self.ui.infoButton.setIcon(QIcon(qta.icon("fa5s.info-circle").pixmap(QSize(20, 20))))
        self.ui.mainTabWidget.setTabIcon(0, qta.icon("ph.book-open"))
        self.ui.mainTabWidget.setTabIcon(1, qta.icon("ph.book-bookmark"))
        self.ui.mainTabWidget.setTabIcon(2, qta.icon("ph.desktop-tower"))

        center = QScreen.availableGeometry(QApplication.primaryScreen()).center()
        geo = self.frameGeometry()
        geo.moveCenter(center)
        self.move(geo.topLeft())

        self.timer.timeout.connect(self.update_requests)
        self.timer.start(1000)

    def load_repositories(self):
        """
        Loads the user's repositories.
        """
        repositories_widget = Repositories(self.user)
        self.ui.repositoriesTab.setLayout(QVBoxLayout(self.ui.repositoriesTab).addWidget(repositories_widget))

    def load_local_repositories(self):
        """
        Loads the user's local repositories.
        """
        local_repositories_widget = LocalRepositories(self.user)
        self.ui.localTab.setLayout(QVBoxLayout(self.ui.localTab).addWidget(local_repositories_widget))

    def load_overview(self):
        """
        Loads the user's overview.
        """
        overview_widget = Overview(self.user)
        self.ui.overviewTab.setLayout(QVBoxLayout(self.ui.overviewTab).addWidget(overview_widget))

    def load_user_info(self):
        """
        Loads the user's information.
        """
        user_data = self.user.get_named_user_data()
        if user_data.name:
            self.ui.nameLabel.setText(user_data.name)
        else:
            self.ui.nameLabel.hide()

        if user_data.login:
            self.ui.userNameLabel.setText(user_data.login)
        else:
            self.ui.userNameLabel.hide()

        if user_data.bio:
            self.ui.bioLabel.setText(user_data.bio)
        else:
            self.ui.bioLabel.hide()

        if user_data.location:
            self.ui.locationWidget.setText(user_data.location)
            self.ui.locationWidget.set_icon("fa5s.map-marker-alt")
        else:
            self.ui.locationWidget.hide()

        if user_data.company:
            self.ui.companyWidget.setText(user_data.company)
            self.ui.companyWidget.set_icon("fa5s.building")
        else:
            self.ui.companyWidget.hide()

        if user_data.twitter_username:
            self.ui.twitterWidget.setText(user_data.twitter_username)
            self.ui.twitterWidget.set_icon("fa5b.twitter")
        else:
            self.ui.twitterWidget.hide()

        if user_data.created_at:
            self.ui.joinedWidget.setText(time_formatter("Joined", time2=user_data.created_at))
            self.ui.joinedWidget.set_icon("fa5s.clock")
        else:
            self.ui.joinedWidget.hide()

        image = QImage()
        image.loadFromData(requests.get(user_data.avatar_url).content)
        self.ui.profileImageLabel.setPixmap(QPixmap(image).scaled(QSize(120, 120), Qt.KeepAspectRatio))

    def reload(self):
        """
        Reloads the user's information.
        """
        self.load_user_info()
        self.load_repositories()
        self.load_local_repositories()

    def update_requests(self):
        """
        Updates the requests.
        """
        user_requests = self.user.github.rate_limiting[0]
        max_requests = self.user.github.rate_limiting[1]
        self.ui.infoButton.setToolTip("Requests: {}/{}".format(user_requests, max_requests))

    def requests_info(self):
        """
        Shows the requests information.
        """
        message = QMessageBox()
        message.setWindowTitle("Requests")
        message.setText("Requests: {}/{} \n{}".format(
            self.user.github.rate_limiting[0],
            self.user.github.rate_limiting[1],
            time_formatter("Reset:", time1=datetime.utcfromtimestamp(self.user.github.rate_limiting_resettime),
                           suffix="")
        ))
        message.setDefaultButton(QMessageBox.Ok)
        message.exec()

    def edit_profile(self):
        """
        Edits the user's profile.
        """
        self.edit = EditProfile(self.user)
        self.edit.show()
        self.load_user_info()

    def closeEvent(self, event):
        """
        Closes the application.

        Closes the application when the user clicks the X button or presses the Exit action.

        Parameters
        ----------
        event : QCloseEvent
            The close event.
        """
        quit()


if __name__ == "__main__":
    app = QApplication([])
    app.setWindowIcon(QIcon(LOGO))
    widget = GitHubManager()
    sys.exit(app.exec())



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'edit_profile.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QMetaObject)
from PySide6.QtGui import (QFont)
from PySide6.QtWidgets import (QHBoxLayout, QLabel, QLineEdit,
                               QPushButton, QSizePolicy, QSpacerItem, QTextEdit,
                               QVBoxLayout)


class Ui_EditProfile(object):
    def setupUi(self, EditProfile):
        if not EditProfile.objectName():
            EditProfile.setObjectName(u"EditProfile")
        EditProfile.resize(400, 321)
        self.verticalLayout = QVBoxLayout(EditProfile)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.label = QLabel(EditProfile)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setPointSize(16)
        font.setBold(True)
        self.label.setFont(font)

        self.verticalLayout.addWidget(self.label)

        self.label_2 = QLabel(EditProfile)
        self.label_2.setObjectName(u"label_2")
        font1 = QFont()
        font1.setBold(True)
        self.label_2.setFont(font1)

        self.verticalLayout.addWidget(self.label_2)

        self.nameLineEdit = QLineEdit(EditProfile)
        self.nameLineEdit.setObjectName(u"nameLineEdit")

        self.verticalLayout.addWidget(self.nameLineEdit)

        self.label_3 = QLabel(EditProfile)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setFont(font1)

        self.verticalLayout.addWidget(self.label_3)

        self.bioTextEdit = QTextEdit(EditProfile)
        self.bioTextEdit.setObjectName(u"bioTextEdit")

        self.verticalLayout.addWidget(self.bioTextEdit)

        self.label_4 = QLabel(EditProfile)
        self.label_4.setObjectName(u"label_4")

        self.verticalLayout.addWidget(self.label_4)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(-1, -1, -1, 0)
        self.companyLabel = QLabel(EditProfile)
        self.companyLabel.setObjectName(u"companyLabel")

        self.horizontalLayout_2.addWidget(self.companyLabel)

        self.companyLineEdit = QLineEdit(EditProfile)
        self.companyLineEdit.setObjectName(u"companyLineEdit")

        self.horizontalLayout_2.addWidget(self.companyLineEdit)

        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalLayout_3.setContentsMargins(-1, -1, -1, 0)
        self.locationLabel = QLabel(EditProfile)
        self.locationLabel.setObjectName(u"locationLabel")

        self.horizontalLayout_3.addWidget(self.locationLabel)

        self.locationLineEdit = QLineEdit(EditProfile)
        self.locationLineEdit.setObjectName(u"locationLineEdit")

        self.horizontalLayout_3.addWidget(self.locationLineEdit)

        self.verticalLayout.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.horizontalLayout_7.setContentsMargins(-1, -1, -1, 0)
        self.saveButton = QPushButton(EditProfile)
        self.saveButton.setObjectName(u"saveButton")

        self.horizontalLayout_7.addWidget(self.saveButton)

        self.cancelButton = QPushButton(EditProfile)
        self.cancelButton.setObjectName(u"cancelButton")

        self.horizontalLayout_7.addWidget(self.cancelButton)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_7.addItem(self.horizontalSpacer)

        self.verticalLayout.addLayout(self.horizontalLayout_7)

        self.retranslateUi(EditProfile)

        QMetaObject.connectSlotsByName(EditProfile)

    # setupUi

    def retranslateUi(self, EditProfile):
        EditProfile.setWindowTitle(QCoreApplication.translate("EditProfile", u"Form", None))
        self.label.setText(QCoreApplication.translate("EditProfile", u"Edit profile", None))
        self.label_2.setText(QCoreApplication.translate("EditProfile", u"Name", None))
        self.nameLineEdit.setPlaceholderText(QCoreApplication.translate("EditProfile", u"Name", None))
        self.label_3.setText(QCoreApplication.translate("EditProfile", u"Bio", None))
        self.bioTextEdit.setPlaceholderText(QCoreApplication.translate("EditProfile", u"Add a bio", None))
        self.label_4.setText(QCoreApplication.translate("EditProfile",
                                                        u"You can @mention other users and organizations to link to them.",
                                                        None))
        self.companyLabel.setText("")
        self.companyLineEdit.setPlaceholderText(QCoreApplication.translate("EditProfile", u"Company", None))
        self.locationLabel.setText("")
        self.locationLineEdit.setPlaceholderText(QCoreApplication.translate("EditProfile", u"Location", None))
        self.saveButton.setText(QCoreApplication.translate("EditProfile", u"Save", None))
        self.cancelButton.setText(QCoreApplication.translate("EditProfile", u"Cancel", None))
    # retranslateUi



import pygit2 as pygit2
from github.AuthenticatedUser import AuthenticatedUser
from github.NamedUser import NamedUser
from github.PaginatedList import PaginatedList


class User:
    def __init__(self, github, user, token):
        """
        Initializes the user object.

        Parameters
        ----------
        github : Github
            Github object.
        user : AuthenticatedUser
            Authenticated user.
        token : str
            Token of the authenticated user.
        """
        self.github = github
        self.user: AuthenticatedUser = user
        self.named_user: NamedUser = self.github.get_user(self.user.login)
        self.repos = None
        self.token = token
        self.pygit_callback = pygit2.RemoteCallbacks(pygit2.UserPass(token, 'x-oauth-basic'))

    def generate_repos(self):
        """
        Generates a list of repositories for the authenticated user.

        Returns
        -------
        bool
            True if the list of repositories was generated successfully.
        """
        check = False
        if self.user is not None:
            self.repos: PaginatedList = self.user.get_repos()
            check = True
        return check

    def get_repos(self):
        """
        Returns the list of repositories for the authenticated user.

        Returns
        -------
        PaginatedList
            List of repositories.
        """
        check = None
        if self.repos is None:
            if self.generate_repos():
                check = self.repos
        else:
            check = self.repos
        return check

    def get_data(self):
        """
        Returns the data of the authenticated user.

        Returns
        -------
        AuthenticatedUser
            Data of the authenticated user.
        """
        return self.user

    def get_named_user_data(self):
        """
        Returns the data of the named user.

        Returns
        -------
        NamedUser
            Data of the named user.
        """
        return self.named_user



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'loginwindow.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QHBoxLayout, QLabel,
    QLineEdit, QMainWindow, QMenu, QMenuBar,
    QPushButton, QSizePolicy, QSpacerItem, QStatusBar,
    QVBoxLayout, QWidget)

from pygithubmanager.widgets.IconLabel import IconLabel

class Ui_GitHubManagerLogin(object):
    def setupUi(self, GitHubManagerLogin):
        if not GitHubManagerLogin.objectName():
            GitHubManagerLogin.setObjectName(u"GitHubManagerLogin")
        GitHubManagerLogin.resize(369, 247)
        self.actionAbout = QAction(GitHubManagerLogin)
        self.actionAbout.setObjectName(u"actionAbout")
        self.actionHow_to_get_an_GitHub_acces_token = QAction(GitHubManagerLogin)
        self.actionHow_to_get_an_GitHub_acces_token.setObjectName(u"actionHow_to_get_an_GitHub_acces_token")
        self.actionExit = QAction(GitHubManagerLogin)
        self.actionExit.setObjectName(u"actionExit")
        self.centralwidget = QWidget(GitHubManagerLogin)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayout = QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.ghmImageLabel = QLabel(self.centralwidget)
        self.ghmImageLabel.setObjectName(u"ghmImageLabel")
        self.ghmImageLabel.setAlignment(Qt.AlignCenter)

        self.verticalLayout.addWidget(self.ghmImageLabel)

        self.signInLabel = QLabel(self.centralwidget)
        self.signInLabel.setObjectName(u"signInLabel")
        font = QFont()
        font.setPointSize(15)
        font.setBold(True)
        self.signInLabel.setFont(font)
        self.signInLabel.setLayoutDirection(Qt.LeftToRight)
        self.signInLabel.setAlignment(Qt.AlignCenter)

        self.verticalLayout.addWidget(self.signInLabel)

        self.label = QLabel(self.centralwidget)
        self.label.setObjectName(u"label")

        self.verticalLayout.addWidget(self.label)

        self.frame = QFrame(self.centralwidget)
        self.frame.setObjectName(u"frame")
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_2 = QHBoxLayout(self.frame)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.tokenLineEdit = QLineEdit(self.frame)
        self.tokenLineEdit.setObjectName(u"tokenLineEdit")

        self.horizontalLayout_2.addWidget(self.tokenLineEdit)

        self.switchVisibilityButton = QPushButton(self.frame)
        self.switchVisibilityButton.setObjectName(u"switchVisibilityButton")

        self.horizontalLayout_2.addWidget(self.switchVisibilityButton)


        self.verticalLayout.addWidget(self.frame)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.signInButton = QPushButton(self.centralwidget)
        self.signInButton.setObjectName(u"signInButton")

        self.horizontalLayout.addWidget(self.signInButton)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.githubAccesTokenButton = QPushButton(self.centralwidget)
        self.githubAccesTokenButton.setObjectName(u"githubAccesTokenButton")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.githubAccesTokenButton.sizePolicy().hasHeightForWidth())
        self.githubAccesTokenButton.setSizePolicy(sizePolicy)
        self.githubAccesTokenButton.setLayoutDirection(Qt.LeftToRight)

        self.horizontalLayout.addWidget(self.githubAccesTokenButton)


        self.verticalLayout.addLayout(self.horizontalLayout)

        self.errorWidget = IconLabel(self.centralwidget)
        self.errorWidget.setObjectName(u"errorWidget")
        sizePolicy.setHeightForWidth(self.errorWidget.sizePolicy().hasHeightForWidth())
        self.errorWidget.setSizePolicy(sizePolicy)

        self.verticalLayout.addWidget(self.errorWidget)

        GitHubManagerLogin.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(GitHubManagerLogin)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 369, 22))
        self.menuMenu = QMenu(self.menubar)
        self.menuMenu.setObjectName(u"menuMenu")
        self.menuHelp = QMenu(self.menubar)
        self.menuHelp.setObjectName(u"menuHelp")
        GitHubManagerLogin.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(GitHubManagerLogin)
        self.statusbar.setObjectName(u"statusbar")
        GitHubManagerLogin.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuMenu.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())
        self.menuMenu.addAction(self.actionExit)
        self.menuHelp.addAction(self.actionHow_to_get_an_GitHub_acces_token)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.actionAbout)

        self.retranslateUi(GitHubManagerLogin)

        QMetaObject.connectSlotsByName(GitHubManagerLogin)
    # setupUi

    def retranslateUi(self, GitHubManagerLogin):
        GitHubManagerLogin.setWindowTitle(QCoreApplication.translate("GitHubManagerLogin", u"Login", None))
        self.actionAbout.setText(QCoreApplication.translate("GitHubManagerLogin", u"About", None))
        self.actionHow_to_get_an_GitHub_acces_token.setText(QCoreApplication.translate("GitHubManagerLogin", u"How to get an GitHub acces token", None))
        self.actionExit.setText(QCoreApplication.translate("GitHubManagerLogin", u"Exit", None))
        self.ghmImageLabel.setText("")
        self.signInLabel.setText(QCoreApplication.translate("GitHubManagerLogin", u"Sign in to GitHub Manager", None))
        self.label.setText(QCoreApplication.translate("GitHubManagerLogin", u"Personal acces token:", None))
        self.tokenLineEdit.setText("")
        self.tokenLineEdit.setPlaceholderText(QCoreApplication.translate("GitHubManagerLogin", u"ghp_W5FU93CFrM5y5a56fsaFFFV1EYaIf", None))
        self.switchVisibilityButton.setText("")
        self.signInButton.setText(QCoreApplication.translate("GitHubManagerLogin", u"Sign in", None))
        self.githubAccesTokenButton.setText(QCoreApplication.translate("GitHubManagerLogin", u"Login with GitHub", None))
        self.menuMenu.setTitle(QCoreApplication.translate("GitHubManagerLogin", u"Menu", None))
        self.menuHelp.setTitle(QCoreApplication.translate("GitHubManagerLogin", u"Help", None))
    # retranslateUi




import qtawesome as qta
from PySide6.QtCore import QSize
from PySide6.QtWidgets import QWidget, QHBoxLayout, QLabel


class IconLabel(QWidget):
    IconSize = QSize(16, 16)
    HorizontalSpacing = 2

    def __init__(self, text, qta_id=None, final_stretch=True):
        """
        Initialize the icon label.

        Parameters
        ----------
        text : str
            The text of the label.
        qta_id : str
            The qta_id of the icon.
        final_stretch : bool
            Whether the label should stretch to the width of the widget.
        """
        super(IconLabel, self).__init__()

        layout = QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(layout)

        self.icon = QLabel()
        if qta_id:
            self.icon.setPixmap(qta.icon(qta_id).pixmap(self.IconSize))

        layout.addWidget(self.icon)
        layout.addSpacing(self.HorizontalSpacing)
        self.label = QLabel(text)
        layout.addWidget(self.label)

        if final_stretch:
            layout.addStretch()

    def set_icon(self, qta_id, color="black", size=IconSize):
        """
        Set the icon to the label with the given qta_id.

        Parameters
        ----------
        qta_id : str
            The qta_id of the icon.
        color : str
            The color of the icon.
        size : QSize
            The size of the icon.
        """
        self.icon.setPixmap(qta.icon(qta_id, color=color).pixmap(size))

    def setText(self, text):
        """
        Set the text of the label.

        Parameters
        ----------
        text : str
            The text of the label.
        """
        self.label.setText(text)

    def set_text_style(self, color=None, font_size=None, bold=False, italic=False, underline=False, font_family=None):
        """
        Set the style of the label.

        Parameters
        ----------
        color : str
            The color of the text.
        font_size : int
            The size of the font.
        bold : bool
            Whether the text should be bold.
        italic : bool
            Whether the text should be italic.
        underline : bool
            Whether the text should be underlined.
        font_family : str
            The font family of the text.
        """
        style = ""
        if color:
            style += "color: %s;" % color
        if font_size:
            style += "font-size: %spx;" % font_size
        if bold:
            style += "font-weight: bold;"
        if italic:
            style += "font-style: italic;"
        if underline:
            style += "text-decoration: underline;"
        if font_family:
            style += "font-family: %s;" % font_family
        self.label.setStyleSheet(style)



from PySide6.QtCore import Signal
from PySide6.QtWidgets import QLabel


class ClickableLabel(QLabel):
    clicked = Signal()

    def __init__(self, *args):
        """
        ClickableLabel to open a repo

        Parameters
        ----------
        args : list
            Arguments to pass to QLabel
        """
        QLabel.__init__(self, *args)

    def mouseReleaseEvent(self, ev):
        """
        Emit clicked signal when mouse is released
        Parameters
        ----------
        ev : QMouseEvent
            Mouse event
        """
        self.clicked.emit()



import sys
import traceback

import qtawesome as qta
import requests
from PySide6.QtCore import QUrl, QSize, QRunnable, Signal, Slot, QObject, QThreadPool
from PySide6.QtGui import QPixmap, QDesktopServices, QIcon, QImage
from PySide6.QtWidgets import QWidget, QListWidgetItem
from github import GithubException
from github.AuthenticatedUser import AuthenticatedUser
from github.Commit import Commit

from pygithubmanager.modules import User
from pygithubmanager.ui.widgets import Ui_repository
from pygithubmanager.utils import time_formatter, message
from pygithubmanager.widgets import CloneTemplate, FileTemplate, RepositoryListWidgetItem, EditRepository, IssueTemplate


class WorkerSignals(QObject):
    file = Signal(str, bool, str, int, str)
    finished = Signal()
    error = Signal(tuple)


class Worker(QRunnable):
    def __init__(self, content):
        """
        Worker to get the content of a repository

        Parameters
        ----------
        content
            The content of the repository
        """
        super(Worker, self).__init__()
        self.content = content
        self.signals = WorkerSignals()

    @Slot(str)
    def run(self):
        """
        Run the worker to get the content of a repository and emit the signals to update the UI
        """
        try:
            while self.content:
                file_content = self.content.pop(0)
                if file_content.type == "dir":
                    self.signals.file.emit(file_content.name, True, file_content.type, file_content.size, "")
                else:
                    self.signals.file.emit(file_content.name, False, file_content.type, file_content.size,
                                           file_content.html_url)
        except Exception:
            traceback.print_exc()
            exctype, value = sys.exc_info()[:2]
            self.signals.error.emit((exctype, value, traceback.format_exc()))
        finally:
            self.signals.finished.emit()


class Repository(QWidget):
    def __init__(self, repo, user):
        """
        Initialize the repository widget

        Parameters
        ----------
        repo : github.Repository.Repository
            The repository object
        user : User
            The user object
        """
        super(Repository, self).__init__()
        self.ui = Ui_repository()
        self.ui.setupUi(self)

        self.repository = repo

        self.user: User = user
        self.authenticated_user: AuthenticatedUser = user.get_data()

        self.clone = CloneTemplate(self.repository, self.user)
        self.edit_repo = QWidget()

        self.paths = [""]

        self.thread_pool = QThreadPool()

        self.load_data()
        self.load_code_data()
        self.load_issues()
        self.config_ui()

    def load_data(self):
        """
        Load repo data to ui widgets

        Raises
        ------
        Exception
            If there is no license
        """
        self.ui.repoLabel.setText(self.repository.full_name)
        self.watch()
        self.fork()
        self.star()

        self.ui.watchButton.clicked.connect(self.watch_button)
        self.ui.forkButton.clicked.connect(self.fork_button)
        self.ui.starButton.clicked.connect(self.star_button)

        try:
            self.ui.licenseWidget.setText(self.repository.get_license().license.name)
        except Exception:
            self.ui.licenseWidget.setText("No license")

    def config_ui(self):
        """
        Config ui widgets
        """
        self.setWindowTitle(self.repository.name)
        self.ui.tabWidget.setTabIcon(0, QIcon(qta.icon("fa5s.code").pixmap(10, 10)))
        self.ui.tabWidget.setTabIcon(1, QIcon(qta.icon("fa5s.dot-circle").pixmap(10, 10)))
        self.ui.commitsWidget.set_icon("fa5s.history")
        self.ui.licenseWidget.set_icon("fa5s.balance-scale")
        self.ui.configButton.setIcon(QIcon(qta.icon("ei.cog").pixmap(20, 20)))

        self.ui.descLabel.setText(self.repository.description)
        self.ui.codeButton.clicked.connect(lambda: self.clone.show())
        self.ui.filesListWidget.itemClicked.connect(self.on_click)
        self.ui.configButton.clicked.connect(self.edit_repo_window)
        self.ui.branchComboBox.setCurrentText(self.repository.default_branch)
        self.ui.branchComboBox.currentTextChanged.connect(self.branch_changed)
        self.branch_changed(self.repository.default_branch)

    def load_code_data(self):
        """
        Load repo data to ui widgets
        """
        self.ui.branchComboBox.clear()
        for branch in self.repository.get_branches():
            self.ui.branchComboBox.addItem(QPixmap(qta.icon("fa5s.code-branch").pixmap(10, 10)), branch.name)

        image = QImage()
        image.loadFromData(requests.get(self.repository.owner.avatar_url).content)
        self.ui.profileImageLabel.setPixmap(QPixmap(image).scaled(QSize(20, 20)))
        self.ui.usernameLabel.setText(self.repository.owner.login)
        try:
            commits = self.repository.get_commits()
            last_commit: Commit = commits[0]
            self.ui.commitLabel.setText(last_commit.commit.message)
            self.ui.numsLabel.setText(last_commit.commit.sha[:7])
            self.ui.timeLabel.setText(time_formatter("", time2=last_commit.commit.author.date))
            self.ui.commitsWidget.setText(
                str(commits.totalCount) + " commit" if commits.totalCount == 1 else str(
                    commits.totalCount) + " commits")
        except GithubException:
            print(GithubException)
            self.ui.commitLabel.setText("No commits")
            self.ui.numsLabel.setText("")
            self.ui.timeLabel.setText("")
            self.ui.commitsWidget.setText("0 commits")

    def load_issues(self):
        """
        Load issues to ui widgets
        """
        issues = self.repository.get_issues()
        for issue in issues:
            item = QListWidgetItem()
            widget = IssueTemplate(issue)
            item.setSizeHint(widget.sizeHint())
            self.ui.issuesListWidget.addItem(item)
            self.ui.issuesListWidget.setItemWidget(item, widget)

    def branch_changed(self, branch_name):
        """
        When branch changed, update files list

        Parameters
        ----------
        branch_name : str
            Branch name
        """
        try:
            content = self.repository.get_contents("/", ref=branch_name)
            self.load_content(content)
        except GithubException as e:
            message("Error", e.data["message"])

    def load_content(self, content):
        """
        Call the function to load content to ui widgets asynchronously and disables some UI elements

        Parameters
        ----------
        content
            Content to load
        """
        self.ui.branchComboBox.setDisabled(True)
        self.ui.configButton.setDisabled(True)
        self.ui.filesListWidget.clear()

        try:
            worker = Worker(content)
            worker.signals.file.connect(self.add_file)
            worker.signals.finished.connect(self.load_finished)

            if len(self.paths) > 1:
                item = RepositoryListWidgetItem("..", True, "", path=self.paths[-2],
                                                contents=self.repository.get_contents(
                                                    self.paths[-2], ref=self.ui.branchComboBox.currentText()))
                item.setText("..")
                item.setIcon(QIcon(qta.icon("fa5s.arrow-up").pixmap(10, 10)))
                self.ui.filesListWidget.addItem(item)

            self.thread_pool.start(worker)
        except GithubException as e:
            print(e)
            message('error', e.data['message'])

    def add_file(self, file_name, is_directory, content_type, content_size, url):
        """
        Add file to ui widgets

        Parameters
        ----------
        file_name : str
            File name
        is_directory : bool
            True if file is directory else False
        content_type : str
            File content type (e.g. dir, file, symlink)
        content_size : int
            File content size
        url : str
            GitHub file url

        Raises
        ------
        Exception
            If there is an error loading file
        """
        try:
            if is_directory:
                path = self.paths[-1] + "/" + file_name
                item = RepositoryListWidgetItem(
                    name=file_name, is_directory=True, url="", path=path,
                    contents=self.repository.get_contents(path, ref=self.ui.branchComboBox.currentText()))
            else:
                item = RepositoryListWidgetItem(name=file_name, is_directory=False, url=url)
            widget = FileTemplate(file_name, content_type, content_size)
            item.setSizeHint(widget.sizeHint())
            self.ui.filesListWidget.addItem(item)
            self.ui.filesListWidget.setItemWidget(item, widget)
        except Exception as e:
            message('error', e)

    def load_finished(self):
        """
        When loading finished, enables some UI elements
        """
        self.ui.branchComboBox.setDisabled(False)
        self.ui.configButton.setDisabled(False)

    def on_click(self, item):
        """
        When item clicked, update content if it is directory or open file if it is file

        Parameters
        ----------
        item : RepositoryListWidgetItem
            Item clicked
        """
        if item.isDirectory:
            if item.name == "..":
                self.paths.pop()
                self.load_content(item.contents)
            else:
                self.paths.append(item.path)
                self.load_content(item.contents)
        else:
            QDesktopServices.openUrl(QUrl(item.url))

    def edit_repo_window(self):
        """
        Open edit repository window
        """
        self.edit_repo = EditRepository(self.repository, self)
        self.edit_repo.show()

    def watch(self):
        """
        People subscribed to this repo
        """
        self.ui.watchButton.setText("Unwatch " + str(self.repository.subscribers_count)
                                    if self.repository in self.authenticated_user.get_watched()
                                    else "Watch " + str(self.repository.subscribers_count))
        self.ui.watchButton.setIcon(qta.icon("ei.eye-close") if self.repository in self.authenticated_user.get_watched()
                                    else qta.icon("ei.eye-open"))

        self.ui.watchingWidget.setText(str(self.repository.subscribers_count) + " watching")
        self.ui.watchingWidget.set_icon("fa5s.eye")

    def watch_button(self):
        """
        Watch/Unwatch repository
        """
        if self.repository in self.authenticated_user.get_watched():
            self.authenticated_user.remove_from_watched(self.repository)
        else:
            self.authenticated_user.add_to_watched(self.repository)
        self.watch()

    def fork(self):
        """
        Fork repository and forks count
        """
        if self.repository in self.authenticated_user.get_repos():
            self.ui.forkButton.setDisabled(True)
            self.ui.forkButton.setToolTip("You own this repository")

        self.ui.forkButton.setText("Fork " + str(self.repository.forks_count))
        self.ui.forkButton.setIcon(QPixmap(qta.icon("fa5s.code-branch").pixmap(15, 15)))

        self.ui.forksWidget.setText(
            str(self.repository.forks_count) + " fork" if self.repository.forks_count == 1 else
            str(self.repository.forks_count) + " forks")
        self.ui.forksWidget.set_icon("fa5s.code-branch")

    def fork_button(self):
        """
        Fork repository
        """
        self.authenticated_user.create_fork(self.repository)
        self.fork()

    def star(self):
        """
        Star repository and stars count
        """
        self.ui.starButton.setText("Starred " + str(self.repository.stargazers_count)
                                   if self.repository in self.authenticated_user.get_starred()
                                   else "Star " + str(self.repository.stargazers_count))
        self.ui.starButton.setIcon(QPixmap(
            qta.icon("fa5s.star", color='orange').pixmap(15, 15)
            if self.repository in self.authenticated_user.get_starred()
            else qta.icon("fa5s.star").pixmap(15, 15)))

        self.ui.starWidget.setText(
            str(self.repository.stargazers_count) + " star" if self.repository.stargazers_count == 1 else str(
                self.repository.stargazers_count) + " stars")
        self.ui.starWidget.set_icon("fa5s.star")

    def star_button(self):
        """
        Star repository
        """
        if self.repository in self.authenticated_user.get_starred():
            self.authenticated_user.remove_from_starred(self.repository)
        else:
            self.authenticated_user.add_to_starred(self.repository)
        self.star()



import qtawesome as qta
from PySide6.QtCore import QRunnable, Signal, Slot, QThreadPool, QObject
from PySide6.QtWidgets import QWidget, QListWidgetItem
from github.Repository import Repository as GithubRepository

from pygithubmanager.modules.create_repository import CreateRepository
from pygithubmanager.modules.repository import Repository
from pygithubmanager.ui import Ui_Repositories
from pygithubmanager.widgets import RepositoryTemplate


class WorkerSignals(QObject):
    repo = Signal(GithubRepository)
    error = Signal(tuple)
    finished = Signal()


class Worker(QRunnable):
    def __init__(self, user):
        """
        Worker to get all repositories of a user from Github API and emit them to the UI asynchronously

        Parameters
        ----------
        user : User
            User class instance
        """
        super().__init__()
        self.user = user
        self.signals = WorkerSignals()

    @Slot()
    def run(self):
        for repo in self.user.get_repos():
            self.signals.repo.emit(repo)
        self.signals.finished.emit()


class Repositories(QWidget):
    def __init__(self, user):
        """
        Class to show the repositories of a user

        Parameters
        ----------
        user : User
            User class instance
        """
        super(Repositories, self).__init__()
        self.ui = Ui_Repositories()
        self.ui.setupUi(self)

        self.user = user
        self.open_repo = None
        self.create_repo = None
        self.worker = Worker(self.user)

        self.threadpool = QThreadPool()

        self.config()
        self.async_load_items()

    def async_load_items(self):
        """
        Loads the repositories of the user in the listWidget asynchronously
        """
        self.worker.signals.repo.connect(self.add_repo)
        self.worker.signals.finished.connect(self.finished)
        self.ui.lineEdit.setDisabled(True)
        self.ui.newButton.setDisabled(True)
        self.threadpool.start(self.worker)

    @Slot(GithubRepository)
    def add_repo(self, repo):
        """
        Adds a repository to the listWidget

        Parameters
        ----------
        repo : GithubRepository
            Repository class instance
        """
        item = QListWidgetItem()
        item.setToolTip(repo.name)
        widget = RepositoryTemplate(repo, self.user.get_data())
        item.setSizeHint(widget.sizeHint())
        self.ui.listWidget.addItem(item)
        self.ui.listWidget.setItemWidget(item, widget)

    def finished(self):
        """
        Called when the asynchronous loading of the repositories is finished
        """
        self.ui.lineEdit.setEnabled(True)
        self.ui.newButton.setEnabled(True)

    def config(self):
        """
        Configures the signals of the widget
        """
        self.ui.listWidget.clicked.connect(self.open_repository)
        self.ui.newButton.setIcon(qta.icon('ph.book-bookmark', color='black'))
        self.ui.newButton.clicked.connect(self.create_repository)
        self.ui.newButton.setToolTip("Create a new repository")
        self.ui.lineEdit.textChanged.connect(self.on_text_changed)

    def create_repository(self):
        """
        Opens a new window to create a new repository
        """
        self.create_repo = CreateRepository(self.user)
        self.create_repo.show()

    def open_repository(self):
        """
        Opens the repository selected in a new window
        """
        item = self.ui.listWidget.currentItem()
        repo_name = item.toolTip()
        self.open_repo = Repository(self.user.user.get_repo(repo_name), self.user)
        self.open_repo.show()

    def on_text_changed(self, text):
        """
        Filters the repositories with the text entered the lineEdit

        Parameters
        ----------
        text : str
            Text entered in the lineEdit
        """
        for row in range(self.ui.listWidget.count()):
            item = self.ui.listWidget.item(row)
            widget = self.ui.listWidget.itemWidget(item)
            if text:
                item.setHidden(not widget.repo.name.lower().startswith(text.lower()))
            else:
                item.setHidden(False)



from PySide6.QtWidgets import QMessageBox


def delete_repository_dialog(source):
    """
    Dialog to confirm deletion of a repository.

    Parameters
    ----------
    source : str
        The source of the repository.
        Local: The repository is local.
        Remote: The repository is remote.

    Returns
    -------
    bool
        True if user confirms deletion, False otherwise

    Examples
    --------
    >>> delete_repository_dialog('local')
    True
    >>> delete_repository_dialog('remote')
    False
    """
    res = False
    msg = QMessageBox()
    msg.setIcon(QMessageBox.Warning)
    if source.lower() == 'local':
        msg.setText("You are about to delete the repository from the disk, this action cannot be undone.")
    elif source.lower() == 'remote':
        msg.setText("You are about to delete the repository from GitHub, this action cannot be undone.")
    msg.setInformativeText("Are you sure you want to continue?")
    msg.setWindowTitle("Warning")
    msg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
    response = msg.exec()
    if response == QMessageBox.Yes:
        double_check = QMessageBox()
        double_check.setIcon(QMessageBox.Warning)
        double_check.setText("Are you really sure you want to continue?")
        double_check.setInformativeText("This action cannot be undone.")
        double_check.setWindowTitle("Warning")
        double_check.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        response2 = double_check.exec()
        if response2 == QMessageBox.Yes:
            res = True
    return res


def message(type_, msg, additional_info=None):
    """
    Show a message box with the given type and message.

    Parameters
    ----------
    type_ : str
        The type of message box.
    msg : str
        The message to be displayed.
    additional_info : str, optional
        Additional information to be displayed.

    Returns
    -------
    True if the user clicked the Yes button, False if the user clicked the No button,
    or None if the user clicked the OK button.

    Examples
    --------
    >>> message('error', 'An error occurred.', 'Error message.')
    None
    >>> message('warning', 'A warning occurred.', 'Warning message.')
    True / False
    >>> message('success', 'An information message.')
    None
    """
    messagebox = QMessageBox()
    if type_.lower() == 'error':
        messagebox.setIcon(QMessageBox.Critical)
        messagebox.setWindowTitle('Error')
        messagebox.setStandardButtons(QMessageBox.Ok)
    elif type_.lower() == 'warning':
        messagebox.setIcon(QMessageBox.Warning)
        messagebox.setWindowTitle('Warning')
        messagebox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
    elif type_.lower() == 'success':
        messagebox.setIcon(QMessageBox.Information)
        messagebox.setWindowTitle('Success')
        messagebox.setStandardButtons(QMessageBox.Ok)

    messagebox.setText(msg)
    if additional_info:
        messagebox.setDetailedText(additional_info)

    result = messagebox.exec()
    if result == QMessageBox.Yes:
        ret = True
    elif result == QMessageBox.No:
        ret = False
    else:
        ret = None

    return ret



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'repository_template.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QMetaObject)
from PySide6.QtWidgets import (QHBoxLayout, QLabel, QPushButton,
                               QSizePolicy, QSpacerItem, QVBoxLayout)

from pygithubmanager.widgets.IconLabel import IconLabel


class Ui_RepositoryTemplate(object):
    def setupUi(self, RepositoryTemplate):
        if not RepositoryTemplate.objectName():
            RepositoryTemplate.setObjectName(u"RepositoryTemplate")
        RepositoryTemplate.resize(505, 110)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(RepositoryTemplate.sizePolicy().hasHeightForWidth())
        RepositoryTemplate.setSizePolicy(sizePolicy)
        self.verticalLayout_2 = QVBoxLayout(RepositoryTemplate)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.repoNameLabel = QLabel(RepositoryTemplate)
        self.repoNameLabel.setObjectName(u"repoNameLabel")

        self.horizontalLayout.addWidget(self.repoNameLabel)

        self.visibilityLabel = QLabel(RepositoryTemplate)
        self.visibilityLabel.setObjectName(u"visibilityLabel")

        self.horizontalLayout.addWidget(self.visibilityLabel)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.starButton = QPushButton(RepositoryTemplate)
        self.starButton.setObjectName(u"starButton")

        self.horizontalLayout.addWidget(self.starButton)

        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.forkWidget = IconLabel(RepositoryTemplate)
        self.forkWidget.setObjectName(u"forkWidget")
        sizePolicy1 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.forkWidget.sizePolicy().hasHeightForWidth())
        self.forkWidget.setSizePolicy(sizePolicy1)

        self.verticalLayout_2.addWidget(self.forkWidget)

        self.repoDescLabel = QLabel(RepositoryTemplate)
        self.repoDescLabel.setObjectName(u"repoDescLabel")

        self.verticalLayout_2.addWidget(self.repoDescLabel)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.licenseIconLabel = QLabel(RepositoryTemplate)
        self.licenseIconLabel.setObjectName(u"licenseIconLabel")

        self.horizontalLayout_2.addWidget(self.licenseIconLabel)

        self.licenseNameLabel = QLabel(RepositoryTemplate)
        self.licenseNameLabel.setObjectName(u"licenseNameLabel")

        self.horizontalLayout_2.addWidget(self.licenseNameLabel)

        self.horizontalSpacer_3 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer_3)

        self.updatedLabel = QLabel(RepositoryTemplate)
        self.updatedLabel.setObjectName(u"updatedLabel")

        self.horizontalLayout_2.addWidget(self.updatedLabel)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer_2)

        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.retranslateUi(RepositoryTemplate)

        QMetaObject.connectSlotsByName(RepositoryTemplate)

    # setupUi

    def retranslateUi(self, RepositoryTemplate):
        RepositoryTemplate.setWindowTitle(QCoreApplication.translate("RepositoryTemplate", u"Form", None))
        self.repoNameLabel.setText(QCoreApplication.translate("RepositoryTemplate", u"repo-name", None))
        self.visibilityLabel.setText(QCoreApplication.translate("RepositoryTemplate", u"public", None))
        self.starButton.setText(QCoreApplication.translate("RepositoryTemplate", u"Star", None))
        self.repoDescLabel.setText(QCoreApplication.translate("RepositoryTemplate", u"repo desc", None))
        self.licenseIconLabel.setText("")
        self.licenseNameLabel.setText(QCoreApplication.translate("RepositoryTemplate", u"License Name", None))
        self.updatedLabel.setText(QCoreApplication.translate("RepositoryTemplate", u"Updated", None))
    # retranslateUi



import re

from PySide6.QtCore import QSize
from PySide6.QtGui import QPixmap, QIcon, QScreen
from PySide6.QtWebEngineWidgets import QWebEngineView
from PySide6.QtWidgets import QMainWindow, QLineEdit, QApplication
from github import Github, BadCredentialsException
from requests import ConnectTimeout

from pygithubmanager.modules import About
from pygithubmanager.modules.user import User
from pygithubmanager.ui import Ui_GitHubManagerLogin
from pygithubmanager.utils import LOGO, get_icon
from pygithubmanager.widgets import CustomWebEnginePage


class Login(QMainWindow):
    def __init__(self, main_window):
        """
        Constructor of the login window class

        Configure the interface of the login window.

        Parameters
        ----------
        main_window:
            Main window of the application
        """
        super(Login, self).__init__()

        self.ui = Ui_GitHubManagerLogin()
        self.ui.setupUi(self)

        self.browser = QMainWindow()

        self.about = About()

        self.main_window = main_window

        self.visible_token = True

        self.config()
        self.config_ui()

    def config_ui(self):
        """
        Configure the interface of the login window
        """
        self.setWindowIcon(QIcon(LOGO))
        self.browser.setWindowIcon(QIcon(LOGO))
        self.browser.setWindowTitle("Get personal access token")
        self.ui.ghmImageLabel.setPixmap(QPixmap(LOGO).scaled(QSize(120, 120)))
        self.ui.switchVisibilityButton.setIcon(QIcon(get_icon("dark_ocultar.png")))  # Al modo oscuro
        self.ui.errorWidget.set_icon("fa5s.exclamation-triangle")
        self.ui.errorWidget.set_text_style(color="red", font_size=13, bold=True)

        center = QScreen.availableGeometry(QApplication.primaryScreen()).center()
        geo = self.frameGeometry()
        geo.moveCenter(center)
        self.move(geo.topLeft())

    def config(self):
        """
        Configure the login window widgets and signals
        """
        self.ui.switchVisibilityButton.clicked.connect(self.switch_visibility)
        self.ui.githubAccesTokenButton.clicked.connect(self.get_access_token)
        self.ui.signInButton.clicked.connect(self.login)
        self.ui.signInButton.setShortcut("Return")
        self.ui.signInButton.setShortcut("Enter")

        self.ui.tokenLineEdit.textChanged.connect(self.check_text)
        self.ui.signInButton.setDisabled(True)

        self.ui.actionExit.triggered.connect(self.close)
        self.ui.actionAbout.triggered.connect(lambda x: self.about.show())
        self.ui.errorWidget.hide()

    def switch_visibility(self):
        """
        Switch the visibility of the password field and the button to show/hide the password
        """
        if self.visible_token:
            self.ui.tokenLineEdit.setEchoMode(QLineEdit.Password)
            self.ui.switchVisibilityButton.setIcon(QIcon(get_icon("dark_visible.png")))
        else:
            self.ui.tokenLineEdit.setEchoMode(QLineEdit.Normal)
            self.ui.switchVisibilityButton.setIcon(QIcon(get_icon("dark_ocultar.png")))
        self.visible_token = not self.visible_token

    def check_text(self, text):
        """
        Check if the text is correct and enable the button

        Parameters
        ----------
        text: str
            Text to check
        """
        if re.search(r"^[a-zA-Z]+_.*$", text):
            self.ui.signInButton.setEnabled(True)
        else:
            self.ui.signInButton.setDisabled(True)

        self.ui.errorWidget.hide()

    def login(self):
        """
        Check the token and create the Github object and the user object
        """
        try:
            github = Github(self.ui.tokenLineEdit.text())
            user = github.get_user()
            self.main_window.start(User(github, user, self.ui.tokenLineEdit.text()))
            self.hide()
        except BadCredentialsException:
            self.ui.errorWidget.setText("Invalid token")
            self.ui.errorWidget.show()
        except ConnectTimeout:
            self.ui.errorWidget.setText("Connection timeout")
            self.ui.errorWidget.show()
        except Exception as e:
            self.ui.errorWidget.setText("Unknown error")
            self.ui.errorWidget.show()
            print(e)

    def get_access_token(self):
        """
        This method is used to generate a personal access token for the user.
        """
        browser = QWebEngineView()
        browser.setPage(CustomWebEnginePage(self))
        browser.setUrl("https://github.com/settings/tokens/new")
        self.browser.setCentralWidget(browser)
        self.browser.show()

    def closeEvent(self, event):
        """
        Close event

        Parameters
        ----------
        event: QCloseEvent
            Close event
        """
        quit()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'popular_repositories.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QMetaObject)
from PySide6.QtWidgets import (QFrame, QHBoxLayout, QLabel,
                               QSizePolicy, QSpacerItem, QVBoxLayout)

from pygithubmanager.widgets.ClickableLabel import ClickableLabel
from pygithubmanager.widgets.IconLabel import IconLabel


class Ui_PopularRepositories(object):
    def setupUi(self, PopularRepositories):
        if not PopularRepositories.objectName():
            PopularRepositories.setObjectName(u"PopularRepositories")
        PopularRepositories.resize(192, 128)
        self.verticalLayout = QVBoxLayout(PopularRepositories)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.frame = QFrame(PopularRepositories)
        self.frame.setObjectName(u"frame")
        self.frame.setFrameShape(QFrame.Box)
        self.frame.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_2 = QVBoxLayout(self.frame)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.nameLabel = ClickableLabel(self.frame)
        self.nameLabel.setObjectName(u"nameLabel")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nameLabel.sizePolicy().hasHeightForWidth())
        self.nameLabel.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.nameLabel)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.visibilityLabel = QLabel(self.frame)
        self.visibilityLabel.setObjectName(u"visibilityLabel")
        sizePolicy.setHeightForWidth(self.visibilityLabel.sizePolicy().hasHeightForWidth())
        self.visibilityLabel.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.visibilityLabel)

        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.forkWidget = IconLabel(self.frame)
        self.forkWidget.setObjectName(u"forkWidget")
        sizePolicy.setHeightForWidth(self.forkWidget.sizePolicy().hasHeightForWidth())
        self.forkWidget.setSizePolicy(sizePolicy)

        self.verticalLayout_2.addWidget(self.forkWidget)

        self.descLabel = QLabel(self.frame)
        self.descLabel.setObjectName(u"descLabel")

        self.verticalLayout_2.addWidget(self.descLabel)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.langWidget = IconLabel(self.frame)
        self.langWidget.setObjectName(u"langWidget")
        sizePolicy.setHeightForWidth(self.langWidget.sizePolicy().hasHeightForWidth())
        self.langWidget.setSizePolicy(sizePolicy)

        self.horizontalLayout_2.addWidget(self.langWidget)

        self.starsWidget = IconLabel(self.frame)
        self.starsWidget.setObjectName(u"starsWidget")
        sizePolicy.setHeightForWidth(self.starsWidget.sizePolicy().hasHeightForWidth())
        self.starsWidget.setSizePolicy(sizePolicy)

        self.horizontalLayout_2.addWidget(self.starsWidget)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer_2)

        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.verticalLayout.addWidget(self.frame)

        self.retranslateUi(PopularRepositories)

        QMetaObject.connectSlotsByName(PopularRepositories)
    # setupUi

    def retranslateUi(self, PopularRepositories):
        PopularRepositories.setWindowTitle(QCoreApplication.translate("PopularRepositories", u"Form", None))
        self.nameLabel.setText(QCoreApplication.translate("PopularRepositories", u"TextLabel", None))
        self.visibilityLabel.setText(QCoreApplication.translate("PopularRepositories", u"TextLabel", None))
        self.descLabel.setText(QCoreApplication.translate("PopularRepositories", u"TextLabel", None))
    # retranslateUi




from PySide6.QtWidgets import QLabel


class HyperlinkLabel(QLabel):
    def __init__(self, parent=None):
        """
        Label with a clickable link.

        Parameters
        ----------
        parent : QWidget
            Parent widget.
        """
        super().__init__()

        self.link_template = '<a href={0}>{1}</a>'

        self.setOpenExternalLinks(True)
        self.setParent(parent)

    def setText(self, arg__1: str) -> None:
        """
        Set the text of the label.
        Parameters
        ----------
        arg__1 : str
            Text to set.
        """
        super().setText(self.link_template.format(arg__1, arg__1))



import qtawesome as qta
from PySide6.QtWidgets import QWidget, QApplication, QFileDialog, QMessageBox
from github.Repository import Repository

from pygithubmanager.ui.widgets import Ui_Clone
from pygithubmanager.utils import USER_HOME_PATH, clone


class CloneTemplate(QWidget):
    def __init__(self, repository: Repository, user):
        """
        The clone window constructor

        Parameters
        ----------
        repository : Repository
            The repository to clone/obtain the url
        user : User
            The user instance
        """
        super(CloneTemplate, self).__init__()
        self.ui = Ui_Clone()
        self.ui.setupUi(self)

        self.repo: Repository = repository
        self.user = user

        self.config_ui()

        self.config()

        self.fill()

    def fill(self):
        """
        Fill the widget with the repository data
        """
        self.ui.cloneWidget.setText("Clone")
        self.ui.httpsUrlLineEdit.setText(self.repo.clone_url)
        self.ui.sshUrlLineEdit.setText(self.repo.ssh_url)
        self.ui.repoSavePathLineEdit.setText(str(USER_HOME_PATH))

    def config_ui(self):
        """
        Configure the UI
        """
        self.ui.cloneWidget.set_icon("fa5s.terminal")
        self.ui.httpsCopyButton.setIcon(qta.icon("fa5s.copy"))
        self.ui.sshCopyButton.setIcon(qta.icon("fa5s.copy"))

    def config(self):
        """
        Configure the buttons
        """
        self.ui.httpsCopyButton.clicked.connect(
            lambda: QApplication.clipboard().setText(self.ui.httpsUrlLineEdit.text()))
        self.ui.sshCopyButton.clicked.connect(lambda: QApplication.clipboard().setText(self.ui.sshUrlLineEdit.text()))
        self.ui.cloneButton.clicked.connect(self.clone_repo)
        self.ui.selectDirectoryButton.clicked.connect(self.save_path)

    def save_path(self):
        """
        Repository save path
        """
        new_route = QFileDialog.getExistingDirectory(self, "Select folder", str(USER_HOME_PATH))
        if new_route:
            self.ui.repoSavePathLineEdit.setText(new_route)

    def clone_repo(self):
        """
        Clone the repository
        """
        res = clone(self.repo.clone_url, self.ui.repoSavePathLineEdit.text(), self.user)
        self.res_dialog(res[0], res[1])

    def res_dialog(self, res, message):
        """
        Show the result of the clone

        Parameters
        ----------
        res : tuple
            The result of the clone
        message : str
            The message to show
        """
        message_box = QMessageBox()
        message_box.setIcon(QMessageBox.Information if res else QMessageBox.Critical)
        message_box.setText("Repository cloned successfully" if res else "Failed to clone repository")
        if not res:
            message_box.setInformativeText(message)
        message_box.setWindowTitle("Error" if not res else "Success")
        message_box.setStandardButtons(QMessageBox.Ok)
        message_box.exec()



import qtawesome as qta
from PySide6.QtCore import QUrl
from PySide6.QtGui import QIcon, QDesktopServices
from PySide6.QtWidgets import QWidget

from pygithubmanager.ui.widgets import Ui_LocalRepositoryTemplate
from pygithubmanager.utils import get_repo_info


class LocalRepositoryTemplate(QWidget):
    def __init__(self, repo_path, user):
        """
        Template for a local repository

        Parameters
        ----------
        repo_path : str
            Path to the repository
        user : User
            User instance
        """
        super(LocalRepositoryTemplate, self).__init__()
        self.ui = Ui_LocalRepositoryTemplate()
        self.ui.setupUi(self)

        self.repo_path = repo_path
        self.user = user

        self.repo_user, self.repo_name, self.full_repo_name = None, None, None

        self.config_ui()

        self.fill()

    def fill(self):
        """
        Fill the template with the information of the repository
        """
        self.repo_user, self.repo_name, self.full_repo_name = get_repo_info(self.repo_path)
        self.ui.repoNameLabel.setText(self.repo_name)
        self.ui.pathLineEdit.setText(self.repo_path)

        try:
            repo = self.user.github.get_repo(self.full_repo_name)
            self.ui.githubButton.setIcon(QIcon(qta.icon('msc.github', color='green')))
            self.ui.githubButton.setToolTip('Open in Browser')
            self.ui.githubButton.clicked.connect(lambda: QDesktopServices.openUrl(QUrl(repo.html_url)))
        except Exception:
            self.ui.githubButton.setIcon(QIcon(qta.icon('msc.github', color='red')))
            self.ui.githubButton.setToolTip('Repository not found in GitHub')

    def config_ui(self):
        """
        Configure the UI
        """
        self.ui.pathLineEdit.setDisabled(True)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'edit_repository.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QMetaObject)
from PySide6.QtGui import (QFont)
from PySide6.QtWidgets import (QCheckBox, QComboBox, QFormLayout,
                               QHBoxLayout, QLabel, QLayout, QLineEdit,
                               QPushButton, QSizePolicy, QSpacerItem, QVBoxLayout)

class Ui_RepositoryEdit(object):
    def setupUi(self, RepositoryEdit):
        if not RepositoryEdit.objectName():
            RepositoryEdit.setObjectName(u"RepositoryEdit")
        RepositoryEdit.resize(400, 261)
        self.verticalLayout = QVBoxLayout(RepositoryEdit)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.label = QLabel(RepositoryEdit)
        self.label.setObjectName(u"label")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        font = QFont()
        font.setPointSize(16)
        font.setBold(True)
        self.label.setFont(font)

        self.verticalLayout.addWidget(self.label)

        self.formLayout = QFormLayout()
        self.formLayout.setObjectName(u"formLayout")
        self.nameLabel = QLabel(RepositoryEdit)
        self.nameLabel.setObjectName(u"nameLabel")

        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.nameLabel)

        self.nameLineEdit = QLineEdit(RepositoryEdit)
        self.nameLineEdit.setObjectName(u"nameLineEdit")

        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.nameLineEdit)

        self.descriptionLabel = QLabel(RepositoryEdit)
        self.descriptionLabel.setObjectName(u"descriptionLabel")

        self.formLayout.setWidget(1, QFormLayout.LabelRole, self.descriptionLabel)

        self.descriptionLineEdit = QLineEdit(RepositoryEdit)
        self.descriptionLineEdit.setObjectName(u"descriptionLineEdit")

        self.formLayout.setWidget(1, QFormLayout.FieldRole, self.descriptionLineEdit)

        self.homePageLabel = QLabel(RepositoryEdit)
        self.homePageLabel.setObjectName(u"homePageLabel")

        self.formLayout.setWidget(2, QFormLayout.LabelRole, self.homePageLabel)

        self.homePageLineEdit = QLineEdit(RepositoryEdit)
        self.homePageLineEdit.setObjectName(u"homePageLineEdit")

        self.formLayout.setWidget(2, QFormLayout.FieldRole, self.homePageLineEdit)

        self.privateLabel = QLabel(RepositoryEdit)
        self.privateLabel.setObjectName(u"privateLabel")

        self.formLayout.setWidget(3, QFormLayout.LabelRole, self.privateLabel)

        self.privateCheckBox = QCheckBox(RepositoryEdit)
        self.privateCheckBox.setObjectName(u"privateCheckBox")

        self.formLayout.setWidget(3, QFormLayout.FieldRole, self.privateCheckBox)

        self.defaultBranchLabel = QLabel(RepositoryEdit)
        self.defaultBranchLabel.setObjectName(u"defaultBranchLabel")

        self.formLayout.setWidget(4, QFormLayout.LabelRole, self.defaultBranchLabel)

        self.defaultBranchComboBox = QComboBox(RepositoryEdit)
        self.defaultBranchComboBox.setObjectName(u"defaultBranchComboBox")

        self.formLayout.setWidget(4, QFormLayout.FieldRole, self.defaultBranchComboBox)

        self.verticalLayout.addLayout(self.formLayout)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setSizeConstraint(QLayout.SetDefaultConstraint)
        self.deleteButton = QPushButton(RepositoryEdit)
        self.deleteButton.setObjectName(u"deleteButton")
        sizePolicy.setHeightForWidth(self.deleteButton.sizePolicy().hasHeightForWidth())
        self.deleteButton.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.deleteButton)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.acceptButton = QPushButton(RepositoryEdit)
        self.acceptButton.setObjectName(u"acceptButton")
        sizePolicy.setHeightForWidth(self.acceptButton.sizePolicy().hasHeightForWidth())
        self.acceptButton.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.acceptButton)

        self.cancelButton = QPushButton(RepositoryEdit)
        self.cancelButton.setObjectName(u"cancelButton")
        sizePolicy.setHeightForWidth(self.cancelButton.sizePolicy().hasHeightForWidth())
        self.cancelButton.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.cancelButton)

        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(RepositoryEdit)

        QMetaObject.connectSlotsByName(RepositoryEdit)
    # setupUi

    def retranslateUi(self, RepositoryEdit):
        RepositoryEdit.setWindowTitle(QCoreApplication.translate("RepositoryEdit", u"Edit repository", None))
        self.label.setText(QCoreApplication.translate("RepositoryEdit", u"Edit repository", None))
        self.nameLabel.setText(QCoreApplication.translate("RepositoryEdit", u"Name", None))
        self.descriptionLabel.setText(QCoreApplication.translate("RepositoryEdit", u"Description", None))
        self.homePageLabel.setText(QCoreApplication.translate("RepositoryEdit", u"Home Page", None))
        self.privateLabel.setText(QCoreApplication.translate("RepositoryEdit", u"Private", None))
        self.defaultBranchLabel.setText(QCoreApplication.translate("RepositoryEdit", u"Default Branch", None))
        self.deleteButton.setText(QCoreApplication.translate("RepositoryEdit", u"Delete", None))
        self.acceptButton.setText(QCoreApplication.translate("RepositoryEdit", u"Accept", None))
        self.cancelButton.setText(QCoreApplication.translate("RepositoryEdit", u"Cancel", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'local_repository.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QMetaObject)
from PySide6.QtGui import (QFont)
from PySide6.QtWidgets import (QFormLayout, QHBoxLayout, QLabel, QLineEdit, QPushButton, QSizePolicy,
                               QSpacerItem, QTabWidget, QTreeView, QVBoxLayout,
                               QWidget)

class Ui_localRepository(object):
    def setupUi(self, localRepository):
        if not localRepository.objectName():
            localRepository.setObjectName(u"localRepository")
        localRepository.resize(936, 557)
        self.verticalLayout = QVBoxLayout(localRepository)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.repoLabel = QLabel(localRepository)
        self.repoLabel.setObjectName(u"repoLabel")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.repoLabel.sizePolicy().hasHeightForWidth())
        self.repoLabel.setSizePolicy(sizePolicy)
        font = QFont()
        font.setPointSize(14)
        font.setBold(True)
        self.repoLabel.setFont(font)

        self.verticalLayout.addWidget(self.repoLabel)

        self.tabWidget = QTabWidget(localRepository)
        self.tabWidget.setObjectName(u"tabWidget")
        self.infoTab = QWidget()
        self.infoTab.setObjectName(u"infoTab")
        self.verticalLayout_3 = QVBoxLayout(self.infoTab)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.formLayout = QFormLayout()
        self.formLayout.setObjectName(u"formLayout")
        self.fullNameLabel = QLabel(self.infoTab)
        self.fullNameLabel.setObjectName(u"fullNameLabel")

        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.fullNameLabel)

        self.fullNameLineEdit = QLineEdit(self.infoTab)
        self.fullNameLineEdit.setObjectName(u"fullNameLineEdit")
        self.fullNameLineEdit.setReadOnly(True)

        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.fullNameLineEdit)

        self.ownerLabel = QLabel(self.infoTab)
        self.ownerLabel.setObjectName(u"ownerLabel")

        self.formLayout.setWidget(1, QFormLayout.LabelRole, self.ownerLabel)

        self.ownerLineEdit = QLineEdit(self.infoTab)
        self.ownerLineEdit.setObjectName(u"ownerLineEdit")
        self.ownerLineEdit.setReadOnly(True)

        self.formLayout.setWidget(1, QFormLayout.FieldRole, self.ownerLineEdit)

        self.descriptionLabel = QLabel(self.infoTab)
        self.descriptionLabel.setObjectName(u"descriptionLabel")

        self.formLayout.setWidget(2, QFormLayout.LabelRole, self.descriptionLabel)

        self.descriptionLineEdit = QLineEdit(self.infoTab)
        self.descriptionLineEdit.setObjectName(u"descriptionLineEdit")
        self.descriptionLineEdit.setReadOnly(True)

        self.formLayout.setWidget(2, QFormLayout.FieldRole, self.descriptionLineEdit)

        self.createdAtLabel = QLabel(self.infoTab)
        self.createdAtLabel.setObjectName(u"createdAtLabel")

        self.formLayout.setWidget(3, QFormLayout.LabelRole, self.createdAtLabel)

        self.createdAtLineEdit = QLineEdit(self.infoTab)
        self.createdAtLineEdit.setObjectName(u"createdAtLineEdit")
        self.createdAtLineEdit.setReadOnly(True)

        self.formLayout.setWidget(3, QFormLayout.FieldRole, self.createdAtLineEdit)

        self.updatedLabel = QLabel(self.infoTab)
        self.updatedLabel.setObjectName(u"updatedLabel")

        self.formLayout.setWidget(4, QFormLayout.LabelRole, self.updatedLabel)

        self.updatedLineEdit = QLineEdit(self.infoTab)
        self.updatedLineEdit.setObjectName(u"updatedLineEdit")
        self.updatedLineEdit.setReadOnly(True)

        self.formLayout.setWidget(4, QFormLayout.FieldRole, self.updatedLineEdit)

        self.languageLabel = QLabel(self.infoTab)
        self.languageLabel.setObjectName(u"languageLabel")

        self.formLayout.setWidget(5, QFormLayout.LabelRole, self.languageLabel)

        self.languageLineEdit = QLineEdit(self.infoTab)
        self.languageLineEdit.setObjectName(u"languageLineEdit")
        self.languageLineEdit.setReadOnly(True)

        self.formLayout.setWidget(5, QFormLayout.FieldRole, self.languageLineEdit)

        self.forksLabel = QLabel(self.infoTab)
        self.forksLabel.setObjectName(u"forksLabel")

        self.formLayout.setWidget(6, QFormLayout.LabelRole, self.forksLabel)

        self.forksLineEdit = QLineEdit(self.infoTab)
        self.forksLineEdit.setObjectName(u"forksLineEdit")
        self.forksLineEdit.setReadOnly(True)

        self.formLayout.setWidget(6, QFormLayout.FieldRole, self.forksLineEdit)

        self.starsLabel = QLabel(self.infoTab)
        self.starsLabel.setObjectName(u"starsLabel")

        self.formLayout.setWidget(7, QFormLayout.LabelRole, self.starsLabel)

        self.starsLineEdit = QLineEdit(self.infoTab)
        self.starsLineEdit.setObjectName(u"starsLineEdit")
        self.starsLineEdit.setReadOnly(True)

        self.formLayout.setWidget(7, QFormLayout.FieldRole, self.starsLineEdit)

        self.subscribersLabel = QLabel(self.infoTab)
        self.subscribersLabel.setObjectName(u"subscribersLabel")

        self.formLayout.setWidget(8, QFormLayout.LabelRole, self.subscribersLabel)

        self.subscribersLineEdit = QLineEdit(self.infoTab)
        self.subscribersLineEdit.setObjectName(u"subscribersLineEdit")
        self.subscribersLineEdit.setReadOnly(True)

        self.formLayout.setWidget(8, QFormLayout.FieldRole, self.subscribersLineEdit)


        self.verticalLayout_3.addLayout(self.formLayout)

        self.tabWidget.addTab(self.infoTab, "")
        self.filesTab = QWidget()
        self.filesTab.setObjectName(u"filesTab")
        self.verticalLayout_4 = QVBoxLayout(self.filesTab)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.label_2 = QLabel(self.filesTab)
        self.label_2.setObjectName(u"label_2")
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        font1 = QFont()
        font1.setPointSize(11)
        font1.setBold(True)
        self.label_2.setFont(font1)

        self.verticalLayout_4.addWidget(self.label_2)

        self.treeView = QTreeView(self.filesTab)
        self.treeView.setObjectName(u"treeView")

        self.verticalLayout_4.addWidget(self.treeView)

        self.tabWidget.addTab(self.filesTab, "")
        self.commitsTab = QWidget()
        self.commitsTab.setObjectName(u"commitsTab")
        self.verticalLayout_5 = QVBoxLayout(self.commitsTab)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.commitsImageWidget = QWidget(self.commitsTab)
        self.commitsImageWidget.setObjectName(u"commitsImageWidget")

        self.verticalLayout_2.addWidget(self.commitsImageWidget)

        self.verticalLayout_5.addLayout(self.verticalLayout_2)

        self.tabWidget.addTab(self.commitsTab, "")

        self.verticalLayout.addWidget(self.tabWidget)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.deleteButton = QPushButton(localRepository)
        self.deleteButton.setObjectName(u"deleteButton")
        sizePolicy.setHeightForWidth(self.deleteButton.sizePolicy().hasHeightForWidth())
        self.deleteButton.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.deleteButton)

        self.openBrowserButton = QPushButton(localRepository)
        self.openBrowserButton.setObjectName(u"openBrowserButton")
        sizePolicy.setHeightForWidth(self.openBrowserButton.sizePolicy().hasHeightForWidth())
        self.openBrowserButton.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.openBrowserButton)

        self.openFolderButton = QPushButton(localRepository)
        self.openFolderButton.setObjectName(u"openFolderButton")
        sizePolicy.setHeightForWidth(self.openFolderButton.sizePolicy().hasHeightForWidth())
        self.openFolderButton.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.openFolderButton)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(localRepository)

        self.tabWidget.setCurrentIndex(0)

        QMetaObject.connectSlotsByName(localRepository)
    # setupUi

    def retranslateUi(self, localRepository):
        localRepository.setWindowTitle(QCoreApplication.translate("localRepository", u"Form", None))
        self.repoLabel.setText(QCoreApplication.translate("localRepository", u"repo", None))
        self.fullNameLabel.setText(QCoreApplication.translate("localRepository", u"Full name", None))
        self.ownerLabel.setText(QCoreApplication.translate("localRepository", u"Owner", None))
        self.descriptionLabel.setText(QCoreApplication.translate("localRepository", u"Description", None))
        self.createdAtLabel.setText(QCoreApplication.translate("localRepository", u"Created at", None))
        self.updatedLabel.setText(QCoreApplication.translate("localRepository", u"Updated", None))
        self.languageLabel.setText(QCoreApplication.translate("localRepository", u"Language", None))
        self.forksLabel.setText(QCoreApplication.translate("localRepository", u"Forks", None))
        self.starsLabel.setText(QCoreApplication.translate("localRepository", u"Stars", None))
        self.subscribersLabel.setText(QCoreApplication.translate("localRepository", u"Subscribers", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.infoTab),
                                  QCoreApplication.translate("localRepository", u"Info", None))
        self.label_2.setText(QCoreApplication.translate("localRepository", u"Repository files", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.filesTab),
                                  QCoreApplication.translate("localRepository", u"Files", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.commitsTab),
                                  QCoreApplication.translate("localRepository", u"Commits", None))
        self.deleteButton.setText(QCoreApplication.translate("localRepository", u"Delete", None))
        self.openBrowserButton.setText(QCoreApplication.translate("localRepository", u"Open in GitHub", None))
        self.openFolderButton.setText(QCoreApplication.translate("localRepository", u"Open folder", None))
    # retranslateUi




import os
import shutil

import pygit2
from PySide6.QtWidgets import QMessageBox


def check_save_path(save_path):
    """
    Check if the save path is correct, and you have the necessary permissions

    Parameters
    ----------
    save_path : str
        Path to save the repository

    Returns
    -------
    bool
        True if the path is correct, False if not

    Examples
    --------
    >>> check_save_path("~/repos/")
    >>> check_save_path("/home/user/repos/")
    """

    check = False
    msg_err = ""
    if os.path.exists(save_path):
        if os.path.isdir(save_path):
            if os.access(save_path, os.R_OK | os.W_OK):
                check = True
            else:
                msg_err = "No tienes permisos en el directorio"
        else:
            msg_err = "La ruta especificada no es un directorio"
    else:
        msg_err = "La ruta especificada no existe"
    return check, msg_err


def check_if_repo_exists(repo_path):
    """
    Check if the repository exists

    Parameters
    ----------
    repo_path : str
        Path to the repository

    Returns
    -------
    bool
        True if the repository exists, False if not

    Examples
    --------
    >>> check_if_repo_exists("/home/user/repo")
    >>> check_if_repo_exists("~/repo")
    """
    response = False
    if os.path.exists(repo_path):
        response = True
    return response


def clone(clone_url, clone_path, user):
    """
    Clone the repository to the specified path

    Parameters
    ----------
    clone_url : str
        URL of the repository
    clone_path : str
        Path to save the repository
    user : modules.User
        User to clone the repository

    Returns
    -------
    tuple
        True if the repository was cloned, False if not and error message if the repository was not cloned

    Examples
    --------
    >>> clone("git@github.com:user/repo.git", "~/repos/", user)

    """
    res = True, ""
    repo_name = clone_url.split('.git')[0].split('/')[-1]
    full_path = os.path.join(clone_path, repo_name)
    check1 = check_save_path(clone_path)
    check2 = check_if_repo_exists(full_path)
    if check1[0]:
        if check2:
            message_box = QMessageBox()
            message_box.setText("A folder with the same name as the repository already exists in the directory")
            message_box.setInformativeText("Do you want to overwrite it?")
            message_box.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
            message_box.setDefaultButton(QMessageBox.No)
            response = message_box.exec_()
            if response == QMessageBox.Yes:
                try:
                    shutil.rmtree(full_path, ignore_errors=True)
                    pygit2.clone_repository(clone_url, full_path, callbacks=user.pygit_callback)
                except Exception:
                    res = False, "Failed to clone repository " + repo_name
            else:
                res = False, "A folder with the same name as the repository already exists in the directory"
        else:
            try:
                pygit2.clone_repository(clone_url, full_path, callbacks=user.pygit_callback)
            except Exception as e:
                res = False, "Failed to clone repository " + repo_name
    return res



import qtawesome as qta
from PySide6.QtWidgets import QWidget
from github import GithubException
from github.AuthenticatedUser import AuthenticatedUser

from pygithubmanager.ui.widgets import Ui_EditProfile
from pygithubmanager.utils import message


class EditProfile(QWidget):
    def __init__(self, user):
        """
        EditProfile constructor

        Parameters
        ----------
        user : User
            User instance
        """
        super(EditProfile, self).__init__()

        self.ui = Ui_EditProfile()
        self.ui.setupUi(self)

        self.user: AuthenticatedUser = user.get_data()

        self.fill()
        self.config()

    def fill(self):
        """
        Fill the widgets with the data
        """
        self.ui.nameLineEdit.setText(self.user.name)
        self.ui.bioTextEdit.setText(self.user.bio)
        self.ui.companyLineEdit.setText(self.user.company)
        self.ui.locationLineEdit.setText(self.user.location)
        self.ui.companyLabel.setPixmap(qta.icon('fa5s.building').pixmap(16, 16))
        self.ui.locationLabel.setPixmap(qta.icon('fa5s.map-marker').pixmap(16, 16))

    def config(self):
        """
        Configure the widgets
        """
        self.ui.saveButton.clicked.connect(self.save)
        self.ui.cancelButton.clicked.connect(self.close)

    def save(self):
        """
        Save the data

        Raises
        ------
        Exception
            If there is an error updating the user
        """
        try:
            self.user.edit(self.ui.nameLineEdit.text(), bio=self.ui.bioTextEdit.toPlainText(),
                           company=self.ui.companyLineEdit.text(), location=self.ui.locationLineEdit.text())
            message('success', 'Profile updated successfully')
            self.close()
        except GithubException as e:
            message('error', e.data['message'])



# read version from installed package
import sys

from PySide6.QtGui import QIcon
from PySide6.QtWidgets import QApplication
from pygithubmanager.githubmanager import GitHubManager
from importlib.metadata import version
__version__ = version("pygithubmanager")

from pygithubmanager.utils import LOGO


def start():
    """
    Starts the main program.
    """
    app = QApplication([])
    app.setWindowIcon(QIcon(LOGO))
    widget = GitHubManager()
    sys.exit(app.exec())



from PySide6.QtWidgets import QListWidgetItem


class RepositoryListWidgetItem(QListWidgetItem):
    def __init__(self, name, is_directory, url, path="", contents=None):
        """
        RepositoryListWidgetItem constructor to store the name, is_directory, url, path and contents of a repository.

        Parameters
        ----------
        name : str
            The name of the repository.
        is_directory : bool
            Whether the repository is a directory or not.
        url : str
            The url of the repository.
        path : str
            The path of the repository.
        contents
            The contents of the repository.
        """
        super().__init__()
        self.name = name
        self.isDirectory = is_directory
        self.url = url
        self.contents = contents
        self.path = path



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pygithubmanager',
 'pygithubmanager.modules',
 'pygithubmanager.ui',
 'pygithubmanager.ui.widgets',
 'pygithubmanager.utils',
 'pygithubmanager.widgets']

package_data = \
{'': ['*'], 'pygithubmanager': ['resources/icons/*']}

install_requires = \
['GitPython>=3.1.27,<4.0.0',
 'PyGithub>=1.55,<2.0',
 'PySide6>=6.3.0,<7.0.0',
 'QtAwesome>=1.1.1,<2.0.0',
 'matplotlib>=3.5.2,<4.0.0',
 'pygit2>=1.9.2,<2.0.0']

setup_kwargs = {
    'name': 'pygithubmanager',
    'version': '1.0.0',
    'description': 'GitHub desktop version created with Python',
    'long_description': '[![Stargazers][stars-shield]][stars-url]\n[![Issues][issues-shield]][issues-url]\n[![MIT License][license-shield]][license-url]\n\n# PyGitHubManager\n\nGitHub desktop version created with Python and PySide6.\n\n## Installation\n\n```bash\n$ pip install pygithubmanager\n```\n\n## Usage\n\n```python\nimport pygithubmanager\n\npygithubmanager.start()\n```\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`githubmanager` was created by DomiTP. It is licensed under the terms of the GNU General Public License v3.0 license.\n\n## Credits\n\n`githubmanager` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n\n<!-- MARKDOWN LINKS & IMAGES -->\n[stars-shield]: https://img.shields.io/github/stars/DomiTP/GitHubManager.svg?style=for-the-badge\n[stars-url]: https://github.com/DomiTP/GitHubManager/stargazers\n[issues-shield]: https://img.shields.io/github/issues/DomiTP/GitHubManager.svg?style=for-the-badge\n[issues-url]: https://github.com/DomiTP/GitHubManager/issues\n[license-shield]: https://img.shields.io/github/license/DomiTP/GitHubManager.svg?style=for-the-badge\n[license-url]: https://github.com/DomiTP/GitHubManager/blob/master/LICENSE.md',
    'author': 'DomiTP',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<3.11',
}


setup(**setup_kwargs)



from datetime import date

import requests
from PySide6.QtCore import Slot
from PySide6.QtWidgets import QWidget, QVBoxLayout
from dateutil.relativedelta import relativedelta
from github.PaginatedList import PaginatedList

from pygithubmanager.ui.widgets import Ui_Overview
from pygithubmanager.widgets import MplCanvas
from pygithubmanager.widgets.PopularRepository import PopularRepository


class Overview(QWidget):
    def __init__(self, user):
        """
        Widgets for the overview tab to show the user's information

        Parameters
        ----------
        user : User
            User object to show the information
        """
        super(Overview, self).__init__()
        self.ui = Ui_Overview()
        self.ui.setupUi(self)

        self.user = user

        self.load_popular_repos()
        self.load_contributions()

    @Slot()
    def load_popular_repos(self):
        """
        Loads the user's most popular repositories
        """
        popular_repos: PaginatedList = self.user.get_data().get_repos(visibility="public", sort="updated",
                                                                      direction="desc")
        positions = [(i, j) for i in range(2) for j in range(2)]
        for positions, popular_repo in zip(positions, popular_repos):
            self.ui.gridLayout.addWidget(PopularRepository(popular_repo, self.user), *positions)

    @Slot()
    def load_contributions(self):
        """
        Loads the user's contributions
        """
        year = date.today().year
        response = requests.get(f'https://skyline.github.com/{self.user.user.login}/{year}.json',
                                auth=(self.user.get_data().login, self.user.token))
        weeks = []
        total = []
        if response.status_code == 200:
            for x in response.json()['contributions']:
                weeks.append((date(year, 1, 1) + relativedelta(weeks=+x['week'])).strftime('%d/%m'))
                acum = 0
                for y in x['days']:
                    acum += int(y['count'])
                total.append(acum)
            canvas = MplCanvas(width=10, height=10, dpi=65, xlabel='WEEKS', ylabel='TOTAL CONTRIBUTIONS',
                               title=f'{year} CONTRIBUTIONS', rotate_xlabel=True)
            canvas.ax.bar(weeks, total)
            self.ui.contributionsWidget.setLayout(QVBoxLayout(self.ui.contributionsWidget).addWidget(canvas))



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'clone.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QHBoxLayout, QLabel, QLineEdit,
    QPushButton, QSizePolicy, QTabWidget, QVBoxLayout,
    QWidget)

from pygithubmanager.widgets.IconLabel import IconLabel

class Ui_Clone(object):
    def setupUi(self, Clone):
        if not Clone.objectName():
            Clone.setObjectName(u"Clone")
        Clone.resize(441, 162)
        self.verticalLayout = QVBoxLayout(Clone)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.cloneWidget = IconLabel(Clone)
        self.cloneWidget.setObjectName(u"cloneWidget")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cloneWidget.sizePolicy().hasHeightForWidth())
        self.cloneWidget.setSizePolicy(sizePolicy)

        self.verticalLayout.addWidget(self.cloneWidget)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.tabWidget = QTabWidget(Clone)
        self.tabWidget.setObjectName(u"tabWidget")
        self.https = QWidget()
        self.https.setObjectName(u"https")
        self.horizontalLayout_3 = QHBoxLayout(self.https)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.httpsUrlLineEdit = QLineEdit(self.https)
        self.httpsUrlLineEdit.setObjectName(u"httpsUrlLineEdit")

        self.horizontalLayout_3.addWidget(self.httpsUrlLineEdit)

        self.httpsCopyButton = QPushButton(self.https)
        self.httpsCopyButton.setObjectName(u"httpsCopyButton")
        sizePolicy.setHeightForWidth(self.httpsCopyButton.sizePolicy().hasHeightForWidth())
        self.httpsCopyButton.setSizePolicy(sizePolicy)

        self.horizontalLayout_3.addWidget(self.httpsCopyButton)

        self.tabWidget.addTab(self.https, "")
        self.ssh = QWidget()
        self.ssh.setObjectName(u"ssh")
        self.horizontalLayout_2 = QHBoxLayout(self.ssh)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.sshUrlLineEdit = QLineEdit(self.ssh)
        self.sshUrlLineEdit.setObjectName(u"sshUrlLineEdit")
        self.sshUrlLineEdit.setReadOnly(True)

        self.horizontalLayout_2.addWidget(self.sshUrlLineEdit)

        self.sshCopyButton = QPushButton(self.ssh)
        self.sshCopyButton.setObjectName(u"sshCopyButton")
        sizePolicy.setHeightForWidth(self.sshCopyButton.sizePolicy().hasHeightForWidth())
        self.sshCopyButton.setSizePolicy(sizePolicy)

        self.horizontalLayout_2.addWidget(self.sshCopyButton)

        self.tabWidget.addTab(self.ssh, "")
        self.cloneTab = QWidget()
        self.cloneTab.setObjectName(u"cloneTab")
        self.verticalLayout_2 = QVBoxLayout(self.cloneTab)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.savePathLabel = QLabel(self.cloneTab)
        self.savePathLabel.setObjectName(u"savePathLabel")
        font = QFont()
        font.setBold(True)
        self.savePathLabel.setFont(font)

        self.verticalLayout_2.addWidget(self.savePathLabel)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.repoSavePathLineEdit = QLineEdit(self.cloneTab)
        self.repoSavePathLineEdit.setObjectName(u"repoSavePathLineEdit")

        self.horizontalLayout_4.addWidget(self.repoSavePathLineEdit)

        self.selectDirectoryButton = QPushButton(self.cloneTab)
        self.selectDirectoryButton.setObjectName(u"selectDirectoryButton")

        self.horizontalLayout_4.addWidget(self.selectDirectoryButton)


        self.verticalLayout_2.addLayout(self.horizontalLayout_4)

        self.cloneButton = QPushButton(self.cloneTab)
        self.cloneButton.setObjectName(u"cloneButton")
        sizePolicy.setHeightForWidth(self.cloneButton.sizePolicy().hasHeightForWidth())
        self.cloneButton.setSizePolicy(sizePolicy)

        self.verticalLayout_2.addWidget(self.cloneButton)

        self.tabWidget.addTab(self.cloneTab, "")

        self.horizontalLayout.addWidget(self.tabWidget)


        self.verticalLayout.addLayout(self.horizontalLayout)


        self.retranslateUi(Clone)

        self.tabWidget.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(Clone)
    # setupUi

    def retranslateUi(self, Clone):
        Clone.setWindowTitle(QCoreApplication.translate("Clone", u"Clone", None))
        self.httpsCopyButton.setText("")
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.https), QCoreApplication.translate("Clone", u"HTTPS", None))
        self.sshCopyButton.setText("")
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.ssh), QCoreApplication.translate("Clone", u"SSH", None))
        self.savePathLabel.setText(QCoreApplication.translate("Clone", u"Save path:", None))
#if QT_CONFIG(tooltip)
        self.repoSavePathLineEdit.setToolTip(QCoreApplication.translate("Clone", u"<html><head/><body><p>Path where\n"
"                                                            the repository will be saved.</p><p>Ex:\n"
"                                                            C:\\Users\\User</p><p>Cloning automatically\n"
"                                                            creates a folder with the name of the repository.</p></body></html>\n"
"                                                        ", None))
#endif // QT_CONFIG(tooltip)
        self.selectDirectoryButton.setText(QCoreApplication.translate("Clone", u"Select directory", None))
        self.cloneButton.setText(QCoreApplication.translate("Clone", u"Clone", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.cloneTab), QCoreApplication.translate("Clone", u"CLONE", None))
    # retranslateUi

